/*
 * Copyright (C) 2001-2006 University Bayreuth, BayCEER, Dept. of Mycology 
 * 
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License 
 * along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package net.metadiversity.diversity.navigator.applet;

import org.w3c.dom.*;                // Core DOM classes
import org.w3c.dom.traversal.*;      // TreeWalker and related DOM classes
import org.apache.xerces.parsers.*;  // Apache Xerces parser classes
import org.xml.sax.*;                // Xerces DOM parser uses some SAX classes
import java.awt.*;
import javax.swing.*;                // Swing classes 
import javax.swing.tree.*;           // TreeModel and related classes
import javax.swing.event.*;          // Tree-related event classes
import java.io.*;                    // For reading the input XML file

public class TreeViewApplet extends javax.swing.JApplet 
{
    /**
     * Comment for <code>serialVersionUID</code>
     */
    private static final long serialVersionUID = 3616727179929728561L;

    Newick nw = null;

    String hostname = null;
    String filename = null;
    
    public TreeViewApplet() 
    {   
        hostname = java.util.ResourceBundle.getBundle("net/metadiversity/diversity/navigator/applet/TreeViewApplet").getString("hostname");        
        filename = java.util.ResourceBundle.getBundle("net/metadiversity/diversity/navigator/applet/TreeViewApplet").getString("filename");
        
        getRootPane().putClientProperty( "defeatSystemEventQueueCheck", Boolean.TRUE );
        try
        {
            nw = new Newick( hostname, filename );
        }
        catch( java.io.IOException ex ) 
        { 
            System.out.println( ex );
        }
        catch( org.xml.sax.SAXException ex )
        { 
            System.out.println( ex );
        }
    }
    
    public void start()
    {
        SwingUtilities.invokeLater( new Runnable() 
        {
            public void run()
            {
                System.out.println( getAppletInfo() );
                getContentPane().setLayout( new java.awt.BorderLayout() );
                JLabel info = new JLabel( hostname + filename );
                getContentPane().add( info, java.awt.BorderLayout.NORTH );
                getContentPane().add( new JScrollPane( nw.getTree() ), java.awt.BorderLayout.CENTER );
                getContentPane().setVisible( true );
                validate();
            }
        });
    }
    
    public class Newick implements TreeModel 
    {
        TreeWalker walker;  // The TreeWalker we're modeling for JTree
        JTree tree;
    
        public JTree getTree()
        {
            return tree;
        }
        /** Create a TreeModel for the specified TreeWalker */
        public Newick( TreeWalker walker ) 
        { 
            this.walker = walker; 
        }
        /** 
         * Create a TreeModel for a TreeWalker that returns all nodes
        * in the specified document
        **/
        public Newick(Document document) 
        {
            DocumentTraversal dt = (DocumentTraversal)document;
            walker = dt.createTreeWalker(document, NodeFilter.SHOW_ALL,null,false);
        }

        /** 
        * Create a TreeModel for a TreeWalker that returns the specified 
        * element and all of its descendant nodes.
        **/
        public Newick(Element element) 
        {
            DocumentTraversal dt = (DocumentTraversal)element.getOwnerDocument();
            walker = dt.createTreeWalker(element, NodeFilter.SHOW_ALL, null,false);
        }

        // Return the root of the tree 
        public Object getRoot() { return walker.getRoot(); }
    
        // Is this node a leaf? (Leaf nodes are displayed differently by JTree)
        public boolean isLeaf(Object node) 
        {
            walker.setCurrentNode((Node)node);   // Set current node
            Node child = walker.firstChild();    // Ask for a child
            return (child == null);              // Does it have any?
        }
    
        // How many children does this node have?
        public int getChildCount(Object node) 
        {
            walker.setCurrentNode((Node)node);   // Set the current node
            // TreeWalker doesn't count children for us, so we count ourselves
            int numkids = 0;
            Node child = walker.firstChild();    // Start with the first child
            while(child != null) {               // Loop 'till there are no more
                numkids++;                       // Update the count
                child = walker.nextSibling();    // Get next child
            }
            return numkids;                      // This is the number of children
        }    
        // Return the specified child of a parent node.
        public Object getChild(Object parent, int index) 
        {
            walker.setCurrentNode( (Node) parent );  // Set the current node
            // TreeWalker provides sequential access to children, not random
            // access, so we've got to loop through the kids one by one
            Node child = walker.firstChild();
            while(index-- > 0) child = walker.nextSibling();
            return child;
        }    
        // Return the index of the child node in the parent node
        public int getIndexOfChild(Object parent, Object child) 
        {
            walker.setCurrentNode( (Node) parent );    // Set current node
            int index = 0;
            Node c = walker.firstChild();           // Start with first child
            while((c != child) && (c != null)) {    // Loop 'till we find a match
        	    index++;
        	    c = walker.nextSibling();           // Get the next child
            }
            return index;                           // Return matching position
        }    
        // Only required for editable trees; unimplemented here.
        public void valueForPathChanged( TreePath path, Object newvalue ) 
        {
        }    
        // This TreeModel never fires any events (since it is not editable)
        // so event listener registration methods are left unimplemented
        public void addTreeModelListener(TreeModelListener l) 
        {
        }
        public void removeTreeModelListener(TreeModelListener l) 
        {
        }
        /**
        * A TreeCellRenderer displays each node of a tree.  The default renderer   
        * displays arbitrary Object nodes by calling their toString() method. 
        * The Component.toString() method returns long strings with extraneous
        * information.  Therefore, we use this "wrapper" implementation of 
        * TreeCellRenderer to convert nodes from Component objects to useful
        * String values before passing those String values on to the default
        * renderer.
        **/
        class ComponentCellRenderer implements TreeCellRenderer 
        {
            TreeCellRenderer renderer;  // The renderer we are a wrapper for
            // Constructor: just remember the renderer
            public ComponentCellRenderer(TreeCellRenderer renderer) 
            {
                this.renderer = renderer;
            }
            // This is the only TreeCellRenderer method.
            // Compute the string to display, and pass it to the wrapped renderer
            public Component getTreeCellRendererComponent( JTree tree, Object value,
						       boolean selected,
						       boolean expanded,
						       boolean leaf, 
                                                       int     row,
						       boolean hasFocus) 
            {
                Node n = (Node) value;            
                String newvalue = value.toString();
                // delete ": null"
                int pos = newvalue.indexOf( ": null" );
                if( pos > 0 )
                newvalue = newvalue.substring( 1, pos );    
                String name = null;                       
                    if( n.getNodeType() == Node.TEXT_NODE )
                    { 
			// Use trim() to strip off leading and trailing space.
			// If nothing is left, then reject the node
			if (((Text)n).getData().trim().length() != 0)
                        {
                            // newvalue = n.getNodeName();
                            newvalue = ( (Text) n).getData().trim();
                        }		    
                    }
                    if( n.hasAttributes() )
                    {
                        Element e = (Element) n;
                        NamedNodeMap attr = e.getAttributes();
                        boolean firstFlag   = true;
                        String  seperator   = ", ";
                        name = "";
                        for( int i= 0; i <  attr.getLength(); i++ )
                        {
                            Node a = attr.item( i );
                            if( firstFlag )
                                firstFlag = false;
                            else
                            name += seperator;    
                            name += a.getNodeName() + "=" + a.getNodeValue();
                        }
                    }
                if( name != null )
                {
                    newvalue += " (" + name + ")"; // unless null
                }
	    // Use the wrapped renderer object to do the real work
	    return renderer.getTreeCellRendererComponent(tree, newvalue,
							 selected, expanded,
							 leaf, row, hasFocus);
	}
    }
        public Newick( String hostname, String filename ) throws IOException, SAXException 
        {
            // Obtain an instance of a Xerces parser to build a DOM tree.
            // Note that we are not using the JAXP API here, so this
            // code uses Apache Xerces APIs that are not standards
            DOMParser parser = new org.apache.xerces.parsers.DOMParser();
            
            InputSource input   = new org.xml.sax.InputSource( "http://" + hostname + filename );

            // Tell the Xerces parser to parse the input source
            parser.parse( input );

            // Ask the parser to give us our DOM Document.  Once we've got the DOM
            // tree, we don't have to use the Apache Xerces APIs any more; from
            // here on, we use the standard DOM APIs
            Document document = parser.getDocument();

            // If we're using a DOM Level 2 implementation, then our Document
            // object ought to implement DocumentTraversal 
            DocumentTraversal traversal = ( DocumentTraversal ) document;

            // For this demonstration, we create a NodeFilter that filters out
            // Text nodes containing only space; these just clutter up the tree
            NodeFilter filter = new NodeFilter() 
            {
		public short acceptNode( Node n ) 
                {
                   
		    if( n.getNodeType() == Node.TEXT_NODE )
                    { 
			// Use trim() to strip off leading and trailing space.
			// If nothing is left, then reject the node
			if (((Text)n).getData().trim().length() == 0)
			    return NodeFilter.FILTER_REJECT;
		    }
/*                   
		    if( n.getNodeType() == Node.ELEMENT_NODE )
                    { 
                        if( n.hasAttributes() )
                        {
//                        org.apache.xerces.dom.AttributeMap ab = n.getAttributes();    
//                        System.out.println ( ab );
                        }
                        // org.w3c.dom.
                        System.out.println ( n.getNodeValue() );
                        System.out.println ( n.getAttributes() );
                    }
                    if( n.hasAttributes() )
                    {
                        Element e = (Element) n;
                        NamedNodeMap attr = e.getAttributes();
                        for( int i= 0; i <  attr.getLength(); i++ )
                        {
                            Node a = attr.item( i );
                            System.out.println( a.getNodeName() + " = " + a.getNodeValue() );
                        }
                    }
 */
		    return NodeFilter.FILTER_ACCEPT;
		}
	    };
            // This set of flags says to "show" all node types except comments
            //	int whatToShow = NodeFilter.SHOW_ALL & ~NodeFilter.SHOW_COMMENT;
            int whatToShow = NodeFilter.SHOW_ALL;
            // Create a TreeWalker using the filter and the flags
            TreeWalker walker = traversal.createTreeWalker( document, whatToShow, filter, false );
            // Instantiate a TreeModel and a JTree to display it
            tree = new JTree( new Newick( walker ) );
            // Renderer
            tree.setCellRenderer( new ComponentCellRenderer( tree.getCellRenderer() ) );        
            tree.putClientProperty( "JTree.lineStyle", "Angled" );
        }
    }
}
